/**************************************************************************
 *
 *	project				 : qtplot
 *
 *	copyright            : (C) 2004 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: $
 *
 *  $Revision: $
 *
 *	$Log: $
 *	
 *
 ***************************************************************************/
/***************************************************************************
 *																		   *
 * This file is part of the qtplot package (a gnuplot plotting frontend    *
 * for Qtopia and Qt)                                                      *
 *																		   *
 * Copyright (C) 2004 by Michael Neuroth.								   *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 ***************************************************************************/

#include "qtplotmainwindowimpl.h"

#include <qcanvas.h>
#include <qaction.h>
#include <qtabwidget.h> 
#include <qtextbrowser.h>
#include <qlineedit.h>
#include <qpainter.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qtextstream.h>

#ifdef ZAURUS
#include "filedialogs.h"
#else
#include <qfiledialog.h>
#include <qprinter.h>
#endif

#include "qtplotconfig.h"

#include <stdlib.h>

#include "sockcomm.h"

// ************************************************************************

class SimpleBitmapCanvas : public QCanvas
{
public:
	SimpleBitmapCanvas( QObject * parent = 0, const char * name = 0 )
	: QCanvas( parent, name ),
	  m_pImage( 0 ),
	  m_bExpandImage( false )
	{
	}

	void SetImagePtr( QImage * pImage )
	{
		m_pImage = pImage;
	}

	void SetExpandFlag( bool bExpandImage )
	{
		m_bExpandImage = bExpandImage;
	}
	bool GetExpandFlag() const
	{
		return m_bExpandImage;
	}

protected:
	virtual void drawBackground( QPainter &, const QRect & area );

private:
	QImage *		m_pImage;
	bool			m_bExpandImage;
};

void SimpleBitmapCanvas::drawBackground( QPainter & aPainter, const QRect & area )
{
	// ** OPTIMIZE HERE: erase is not needed all the time !?
	aPainter.eraseRect( area );

	if( m_pImage )
	{
		if( GetExpandFlag() )
		{
			QImage aScaledImage = m_pImage->smoothScale( width(), height() );
			aPainter.drawImage( 0, 0, aScaledImage );
		}
		else
		{
			aPainter.drawImage( 0, 0, *m_pImage );
		}
	}
}

// ************************************************************************

qtplotMainWindowImpl::qtplotMainWindowImpl( QWidget* parent, const char* name, WFlags f )
	: qtplotMainWindow( parent, name, f ), 
	  m_pImage( 0 ), 
	  m_bDisableLogging( true ),
	  m_sHostname( "localhost" ),
	  m_iPortnumber( QTPLOT_PORT_NO ),
	  m_pServerSocket( 0 )
{
	setCaption("QtPlot");

	// Add your code	
	m_aGraphOpContainer.setAutoDelete( TRUE );

	m_pCanvas = new SimpleBitmapCanvas( m_pGraphicFrame );
	m_pView = new QCanvasView( m_pCanvas, m_pGraphicFrame );

	m_pGraphEnv = new GraphicEnv( m_pCanvas );

	UpdateServerSocket();

	connect( this, SIGNAL(sigActivate()), this, SLOT(sltActivate()) );

#ifdef ZAURUS
	m_pExportFileDlg = new FileDialogS(this, "save",TRUE, WType_Modal, TRUE);
    connect( m_pExportFileDlg, SIGNAL( doneLoad() ), this, SLOT( sltDoImport() ) );
    //connect( m_pExportFileDlg, SIGNAL( done() ), this, SLOT( sltDoExport() ) );
	m_pExportFileDlg->adjustSize();
#endif
}

qtplotMainWindowImpl::~qtplotMainWindowImpl()
{
#ifdef ZAURUS
	delete m_pExportFileDlg;
#endif

	delete m_pServerSocket;

	delete m_pGraphEnv;

	delete m_pImage;
}

void qtplotMainWindowImpl::sltTest()
{
	Update();
}

void qtplotMainWindowImpl::sltClearGraphicElements()
{
	ClearCanvas();

	Update();
}

void qtplotMainWindowImpl::sltEraseGraphicElements()
{
	EraseGraphicElements();

	Update();
}

void qtplotMainWindowImpl::sltDrawGraphicElements()
{
	DrawGraphicElements();

	Update();
}

void qtplotMainWindowImpl::sltOpenFile()
{
#ifdef ZAURUS
	m_pExportFileDlg->setLoad( TRUE );
	m_pExportFileDlg->setDirectory( m_sLastDirectory );
	m_pExportFileDlg->exec();
#else
	QString sFileName = QFileDialog::getOpenFileName();

	if( !sFileName.isEmpty() )
	{
		OpenFile( sFileName );
	
		sltDrawGraphicElements();
	}
#endif
}

void qtplotMainWindowImpl::sltDoImport()
{
#ifdef ZAURUS
	QString * pFileName = m_pExportFileDlg->getAbsSelectedFileName();

	m_sLastDirectory = m_pExportFileDlg->getDirectory();

	OpenFile( *pFileName );

	delete pFileName;
#endif
}

void qtplotMainWindowImpl::sltClearLogging()
{
	m_pLogging->setText( "" ); // clear();
}

void qtplotMainWindowImpl::sltClearImage()
{
	if( m_pImage )
	{
		delete m_pImage;
	}
	m_pImage = 0;

	m_pCanvas->SetImagePtr( m_pImage );

	Update();
}

void qtplotMainWindowImpl::sltAbout()
{
	QMessageBox::about( this, tr("About QtPlot"), 
#ifdef ZAURUS
		tr("QtPlot is a gnuplot plotting\nfrontend for Qtopia and Qt.\n\nVersion 0.2 from 16.5.2004\n\n(C) 2004 by Michael Neuroth") );
#else
		tr("QtPlot is a gnuplot plotting frontend for Qtopia and Qt.\n\nVersion 0.2 from 16.5.2004\n\nCopyright (C) 2004 by Michael Neuroth") );
#endif
}

void qtplotMainWindowImpl::sltExpandImage( bool bNewValue )
{
	m_pCanvas->SetExpandFlag( bNewValue );
	
	Update();
}

void qtplotMainWindowImpl::sltDisableLogging(bool bNewValue )
{
	m_bDisableLogging = bNewValue;
}

void qtplotMainWindowImpl::sltActivate()
{
	show();
	setActiveWindow();
	raise();
}

void qtplotMainWindowImpl::sltConfiguration()
{
	qtplotconfig aDlg(this,"config",TRUE);

	aDlg.m_pHostname->setText( m_sHostname );
	aDlg.m_pPortnumber->setText( QString::number( m_iPortnumber ) );

	int iRet = aDlg.exec();

	if( iRet == 1 )
	{
		m_iPortnumber = aDlg.m_pPortnumber->text().toInt();
		m_sHostname = aDlg.m_pHostname->text();

		UpdateServerSocket();
	}
}

void qtplotMainWindowImpl::sltStartGnuplotShell()
{
#ifdef _MSC_VER
	system( "start wgnuplot" );
#else
	system( "embeddedkonsole gnuplot&" );
#endif
}

void qtplotMainWindowImpl::sltPrint()
{
#ifndef ZAURUS
	QPrinter aPrinter;

	if( aPrinter.setup( this ) )
	{
		QPainter aPainter( &aPrinter );

		QRect rect = aPainter.viewport();
		QSize size = m_pCanvas->size();

		size.scale( rect.size(), QSize::ScaleMin );
		aPainter.setViewport( rect.x(), rect.y(), size.width(), size.height() );
		aPainter.setWindow( m_pCanvas->rect() );
		//aPainter.drawRect( aPainter.window() );
		aPainter.setClipRect( aPainter.viewport() );

		QCanvasItemList items = m_pCanvas->collisions( m_pCanvas->rect() );
		QCanvasItemList::const_iterator it = items.end();
		while( it != items.begin() )
		{
			--it;
			(*it)->draw( aPainter );
		}

		if( m_pImage )
		{
			aPainter.drawImage( 0, 0, *m_pImage );
		}
	}
#endif
}

void qtplotMainWindowImpl::sltNewConnection( ClientSocket * pSocket )
{
	LogLine( tr("New connection to client.\n") );

	connect( pSocket, SIGNAL(sigReceivedText(const QString&)), this, SLOT(sltReceivedText(const QString&)) );
	connect( pSocket, SIGNAL(sigClosedSocket()), this, SLOT(sltClosedClientSocket()) );
	connect( pSocket, SIGNAL(connectionClosed()), SLOT(connectionClosed()) );
}

void qtplotMainWindowImpl::sltReceivedText( const QString & sLine )
{
	LogLine( QString( "Received: " ) + sLine );

	ProcessLine( sLine );
}

void qtplotMainWindowImpl::sltClosedClientSocket()
{
	LogLine( QString( "Closed client socket." ) );
}

void qtplotMainWindowImpl::resizeEvent( QResizeEvent * pEvent )
{
	qtplotMainWindow::resizeEvent( pEvent );

	QRect aFrameRect = m_pGraphicFrame->frameRect();

	// ** update size of the canvas
	m_pView->resize( aFrameRect.width(), aFrameRect.height() );
	m_pCanvas->resize( aFrameRect.width()-5, aFrameRect.height()-5 );

	ClearCanvas();
	DrawGraphicElements();
	Update();
}

void qtplotMainWindowImpl::showEvent( QShowEvent * pShowEvent )
{
	qtplotMainWindow::showEvent( pShowEvent );

	// Trick, damit der Canvas beim ersten Show richtig angezeigt wird
	resize( QSize( size().width()+1, size().height()+1 ) );
	resize( QSize( size().width()-1, size().height()-1 ) );
}

void qtplotMainWindowImpl::AddGraphicElemtent( GraphicOperation * pGraphOp )
{
	m_aGraphOpContainer.append( pGraphOp );
}

void qtplotMainWindowImpl::EraseGraphicElements()
{
	m_aGraphOpContainer.clear();
}

void qtplotMainWindowImpl::DrawGraphicElements()
{
	GraphicOperation * pGrOp = 0;

	for( pGrOp=m_aGraphOpContainer.first(); pGrOp; pGrOp=m_aGraphOpContainer.next() )
	{
		pGrOp->Draw( *m_pGraphEnv );
	}
}

void qtplotMainWindowImpl::ClearCanvas()
{
	GraphicOperation * pGrOp = 0;

	for( pGrOp=m_aGraphOpContainer.first(); pGrOp; pGrOp=m_aGraphOpContainer.next() )
	{
		pGrOp->Clear( *m_pGraphEnv );
	}
}

void qtplotMainWindowImpl::OpenFile( const QString sFileName )
{
	// is it a Image ?
	QImage aTestImage( sFileName );

	if( aTestImage.isNull() )
	{
		// no --> treat file as gnuplot graphic file
		EraseGraphicElements();
		ReadGraphicFile( sFileName );
	}
	else
	{
		// yes --> use qtplot as image viewer
		if( m_pImage )
		{
			delete m_pImage;
		}
		m_pImage = new QImage( aTestImage );
		m_pCanvas->SetImagePtr( m_pImage );
		m_pView->repaintContents( m_pView->contentsX(), m_pView->contentsY(), m_pView->/*contents*/visibleWidth(), m_pView->/*contents*/visibleHeight(), /*bErase*/FALSE );
	}
}

void qtplotMainWindowImpl::ReadGraphicFile( const QString sFileName )
{
	QFile aFile( sFileName );

	if( aFile.exists() )
	{
		aFile.open( IO_ReadOnly );

		QTextStream aTextStream( &aFile );

		while( !aTextStream.atEnd() )
		{
			QString sLine = aTextStream.readLine();

			ProcessLine( sLine );
		}

		aFile.close();
	}
}

void qtplotMainWindowImpl::ProcessLine( const QString & sLine )
{
	GraphicOperation * pGrOp = ParseGraphicFileLine( sLine );

	QString sOutLine;
	if( pGrOp )
	{
		sOutLine = QString( "Ok: " ) + sLine;
	}
	else
	{
		sOutLine = QString( "ERROR: " ) + sLine;
	}
	LogLine( sOutLine );

	if( pGrOp )
	{
		AddGraphicElemtent( pGrOp );
	}
}

GraphicOperation * qtplotMainWindowImpl::ParseGraphicFileLine( const QString & sLine )
{
	GraphicOperation * pGrOp = 0;

	if( sLine.find( "qtd move" )==0 )
	{
		int x = 0;
		int y = 0;

		sscanf( (const char *)sLine, "qtd move x=%d y=%d", &x, &y );
		pGrOp = new GrOpMove( x, y );
	}
	else if( sLine.find( "qtd point" )==0 )
	{
		int x = 0;
		int y = 0;
		int point = 0;

		sscanf( (const char *)sLine, "qtd point x=%d y=%d point=%d", &x, &y, &point );
		pGrOp = new GrOpPoint( x, y, point );
	}
	else if( sLine.find( "qtd vector" )==0 )
	{
		int x = 0;
		int y = 0;

		sscanf( (const char *)sLine, "qtd vector x=%d y=%d", &x, &y );
		pGrOp = new GrOpVector( x, y );
	}
	else if( sLine.find( "qtd linetype" )==0 )
	{
		int linetype = 0;

		sscanf( (const char *)sLine, "qtd linetype type=%d", &linetype );
		pGrOp = new GrOpLinetype( linetype );
	}
	else if( sLine.find( "qtd put_text" )==0 )
	{
		char sBuffer[512];
		int x = 0;
		int y = 0;

		// Bugfix (15.5.2004): text can contain spaces now ! 
		sscanf( (const char *)sLine, "qtd put_text x=%d y=%d str=", &x, &y );
		//strcpy( sBuffer, (const char *)sLine.section("=",3,3) );
		strcpy( sBuffer, (const char *)sLine.right(sLine.length()-sLine.find("str=")-strlen("str=")) );
		pGrOp = new GrOpPutText( x, y, sBuffer );
	}
	else if( sLine.find( "qtd options" )==0 )
	{
		pGrOp = new GrOpOptions();
	}
	else if( sLine.find( "qtd init" )==0 )
	{
		pGrOp = new GrOpInit();
	}
	else if( sLine.find( "qtd graphics" )==0 )
	{
		pGrOp = new GrOpGraphics();

		// before receiving all data, clear the drawing area !
		sltEraseGraphicElements();
	}
	else if( sLine.find( "qtd text" )==0 )
	{
		pGrOp = new GrOpText();

		// update the drawing area to paint the contents
		sltDrawGraphicElements();

		emit sigActivate();
	}
	else if( sLine.find( "qtd reset" )==0 )
	{
		pGrOp = new GrOpReset();

		// close the qtplot application after a reset...
		close();
	}
	
	return pGrOp;
}

void qtplotMainWindowImpl::Update()
{
	m_pView->repaintContents( m_pView->contentsX(), m_pView->contentsY(), m_pView->/*contents*/visibleWidth(), m_pView->/*contents*/visibleHeight(), /*bErase*/FALSE );
	m_pCanvas->update();
}

void qtplotMainWindowImpl::LogLine( const QString & sLine )
{
	if( !m_bDisableLogging )
	{
		m_pLogging->append( sLine );
	}
}

void qtplotMainWindowImpl::UpdateServerSocket()
{
	if( m_pServerSocket )
	{
		delete m_pServerSocket;
		m_pServerSocket = 0;
	}

	m_pServerSocket = new ServerSocket( m_iPortnumber, this );

	connect( m_pServerSocket, SIGNAL(sigNewConnect(ClientSocket*)), SLOT(sltNewConnection(ClientSocket*)) );

	if( !m_pServerSocket->ok() )
	{
		QMessageBox::warning( this, tr("Warning"), tr("Can not bind to requested port !\nPlease use an other port no.") );
	}
}

